@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2010 Red Hat, Inc.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netinet/ether.h>
-#include "wireless-helper.h"
 #include <ctype.h>
 
 #include <glib/gi18n.h>
@@ -323,6 +322,47 @@ none:
 	return NULL;
 }
 
+static void
+check_common_ssid (NMAccessPoint *ap, NMSettingWireless *s_wifi)
+{
+	const char *str_bssid;
+	struct ether_addr *eth_addr;
+	const GByteArray *ssid;
+	GByteArray *bssid;
+	const char *known[] = { "linksys", "o2DSL", "NETGEAR", "WLAN", "ALICE-WLAN" };
+	guint i;
+
+	/* For a certain list of known ESSIDs which are commonly preset by ISPs
+	 * and manufacturers and often unchanged by users, lock the connection
+	 * to the BSSID so that we don't try to auto-connect to your grandma's
+	 * neighbor's WiFi.
+	 *
+	 * NOTE: this list should *not* contain networks that you would like to
+	 * automatically roam to like "Starbucks" or "AT&T" or "T-Mobile HotSpot".
+	 */
+	ssid = nm_access_point_get_ssid (ap);
+	for (i = 0; i < G_N_ELEMENTS (known); i++) {
+		if (   ssid->len == strlen (known[i])
+		    && memcmp (ssid->data, known[i], ssid->len) == 0)
+			break;
+	}
+	if (i == G_N_ELEMENTS (known))
+		return;
+
+	str_bssid = nm_access_point_get_hw_address (ap);
+	if (str_bssid) {
+		eth_addr = ether_aton (str_bssid);
+		if (eth_addr) {
+			bssid = g_byte_array_sized_new (ETH_ALEN);
+			g_byte_array_append (bssid, eth_addr->ether_addr_octet, ETH_ALEN);
+			g_object_set (G_OBJECT (s_wifi),
+			              NM_SETTING_WIRELESS_BSSID, bssid,
+			              NULL);
+			g_byte_array_free (bssid, TRUE);
+		}
+	}
+}
+
 static gboolean
 wireless_new_auto_connection (NMDevice *device,
                               gpointer dclass_data,
@@ -356,9 +396,11 @@ wireless_new_auto_connection (NMDevice *device,
 	mode = nm_access_point_get_mode (info->ap);
 	if (mode == NM_802_11_MODE_ADHOC)
 		g_object_set (s_wireless, NM_SETTING_WIRELESS_MODE, "adhoc", NULL);
-	else if (mode == NM_802_11_MODE_INFRA)
+	else if (mode == NM_802_11_MODE_INFRA) {
 		g_object_set (s_wireless, NM_SETTING_WIRELESS_MODE, "infrastructure", NULL);
-	else
+		/* Lock connection to this AP if it's a manufacturer-default SSID */
+		check_common_ssid (info->ap, s_wireless);
+	} else
 		g_assert_not_reached ();
 
 	dev_caps = nm_device_wifi_get_capabilities (NM_DEVICE_WIFI (device));
@@ -490,6 +532,7 @@ create_new_ap_item (NMDeviceWifi *device,
 	nm_network_menu_item_set_ssid (item, (GByteArray *) ssid);
 
 	dev_caps = nm_device_wifi_get_capabilities (device);
+	nma_icon_check_and_load ("nm-adhoc", &applet->adhoc_icon, applet);
 	nm_network_menu_item_set_detail (item, ap, applet->adhoc_icon, dev_caps);
 	nm_network_menu_item_best_strength (item, nm_access_point_get_strength (ap), applet);
 	nm_network_menu_item_add_dupe (item, ap);
@@ -820,31 +863,26 @@ out:
 	g_slist_free (connections);
 }
 
-static gboolean
+static void
 add_seen_bssid (NMSettingsConnectionInterface *connection, NMAccessPoint *ap)
 {
 	NMSettingWireless *s_wireless;
 	const char *bssid;
 
+	if (!NMA_GCONF_CONNECTION (connection))
+		return;
+
 	s_wireless = NM_SETTING_WIRELESS (nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_WIRELESS));
 	if (!s_wireless)
-		return FALSE;
+		return;
 
 	bssid = nm_access_point_get_hw_address (ap);
 	if (!bssid || !utils_ether_addr_valid (ether_aton (bssid)))
-		return FALSE;
+		return;
 
-	return nm_setting_wireless_add_seen_bssid (s_wireless, bssid);
-}
-
-static void
-bssid_update_cb (NMSettingsConnectionInterface *connection,
-                 GError *error,
-                 gpointer user_data)
-{
-	if (error) {
-		g_warning ("%s: failed to update connection seen BSSIDs: (%d) %s",
-		           __func__, error->code, error->message);
+	if (nm_setting_wireless_add_seen_bssid (s_wireless, bssid)) {
+		/* Ignore secrets since we don't have any here and we're just adding a BSSID */
+		nma_gconf_connection_update (NMA_GCONF_CONNECTION (connection), TRUE);
 	}
 }
 
@@ -877,8 +915,7 @@ notify_active_ap_changed_cb (NMDeviceWifi *device,
 	if (!ssid || !nm_utils_same_ssid (nm_setting_wireless_get_ssid (s_wireless), ssid, TRUE))
 		return;
 
-	if (add_seen_bssid (connection, new))
-		nm_settings_connection_interface_update (connection, bssid_update_cb, NULL);
+	add_seen_bssid (connection, new);
 
 	applet_schedule_update_icon (applet);
 }
@@ -1211,8 +1248,8 @@ wireless_device_state_changed (NMDevice *device,
 
 		/* Save this BSSID to seen-bssids list */
 		connection = applet_get_exported_connection_for_device (device, applet);
-		if (connection && add_seen_bssid (connection, new))
-			nm_settings_connection_interface_update (connection, bssid_update_cb, NULL);
+		if (connection)
+			add_seen_bssid (connection, new);
 	}
 
 	msg = g_strdup_printf (_("You are now connected to the wireless network '%s'."),
@@ -1276,15 +1313,15 @@ wireless_get_icon (NMDevice *device,
 			strength = CLAMP (strength, 0, 100);
 
 			if (strength > 80)
-				pixbuf = applet->wireless_100_icon;
+				pixbuf = nma_icon_check_and_load ("nm-signal-100", &applet->wireless_100_icon, applet);
 			else if (strength > 55)
-				pixbuf = applet->wireless_75_icon;
+				pixbuf = nma_icon_check_and_load ("nm-signal-75", &applet->wireless_75_icon, applet);
 			else if (strength > 30)
-				pixbuf = applet->wireless_50_icon;
+				pixbuf = nma_icon_check_and_load ("nm-signal-50", &applet->wireless_50_icon, applet);
 			else if (strength > 5)
-				pixbuf = applet->wireless_25_icon;
+				pixbuf = nma_icon_check_and_load ("nm-signal-25", &applet->wireless_25_icon, applet);
 			else
-				pixbuf = applet->wireless_00_icon;
+				pixbuf = nma_icon_check_and_load ("nm-signal-00", &applet->wireless_00_icon, applet);
 
 			*tip = g_strdup_printf (_("Wireless network connection '%s' active: %s (%d%%)"),
 			                        id, ssid, strength);
